/*
 * MINICRT.H
 *
 * Header file for minicrt based applications.  Function prototypes and
 * appropriate #defines to default to their use and define Unicode-
 * conditional functions.
 *
 * Copyright (c) 2014 Malcolm J. Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

void *     mini_memcpy(void * dest, void * src, unsigned int len);
int        mini_memcmp(const void * buf1, const void * buf2, unsigned int len);
void *     mini_memset(void * dest, char c, unsigned int len);

int        mini_atoi(char * str);
int        mini_wtoi(wchar_t * str);

char *     mini_strncat(char * dest, const char * src, unsigned int len);
wchar_t *  mini_wcsncat(wchar_t * dest, const wchar_t * src, unsigned int len);

char *     mini_strcat_s(char * dest, unsigned int len, const char * src);
wchar_t *  mini_wcscat_s(wchar_t * dest, unsigned int len, const wchar_t * src);

char *     mini_strchr(const char * str, char ch);
wchar_t *  mini_wcschr(const wchar_t * str, wchar_t ch);

char *     mini_strrchr(const char * str, char ch);
wchar_t *  mini_wcsrchr(const wchar_t * str, wchar_t ch);

char *     mini_strtok_s(char * str, char * match, char ** context);
wchar_t *  mini_wcstok_s(wchar_t * str, wchar_t * match, wchar_t ** context);

char *     mini_strtok(char * str, char * match);
wchar_t *  mini_wcstok(wchar_t * str, wchar_t * match);

int        mini_vsprintf_s(char * szDest, unsigned int len, const char * szFmt, va_list marker);
int        mini_vswprintf_s(wchar_t * szDest, unsigned int len, const wchar_t * szFmt, va_list marker);

int        mini_vsprintf_size(const char * szFmt, va_list marker);
int        mini_vswprintf_size(const wchar_t * szFmt, va_list marker);

int        mini_sprintf_s(char * szDest, unsigned int len, const char * szFmt, ...);
int        mini_swprintf_s(wchar_t * szDest, unsigned int len, const wchar_t * szFmt, ...);

int        mini_sprintf(char * szDest, const char * szFmt, ...);
int        mini_swprintf(wchar_t * szDest, const wchar_t * szFmt, ...);

int        mini_fprintf(int fp, const char * szFmt, ...);
int        mini_fwprintf(int fp, const wchar_t * szFmt, ...);

int        mini_printf(const char * szFmt, ...);
int        mini_wprintf(const wchar_t * szFmt, ...);

//
//  Map tchar versions to the correct place
//

#ifdef UNICODE
#define    mini_ttoi        mini_wtoi
#define    mini_tcsncat     mini_wcsncat
#define    mini_tcscat_s    mini_wcscat_s
#define    mini_tcschr      mini_wcschr
#define    mini_tcsrchr     mini_wcsrchr
#define    mini_tcstok_s    mini_wcstok_s
#define    mini_tcstok      mini_wcstok
#define    mini_vstprintf_s mini_vswprintf_s
#define    mini_vstprintf_size mini_vswprintf_size
#define    mini_stprintf_s  mini_swprintf_s
#define    mini_stprintf    mini_swprintf
#define    mini_ftprintf    mini_fwprintf
#define    mini_tprintf     mini_wprintf
#else
#define    mini_ttoi        mini_atoi
#define    mini_tcsncat     mini_strncat
#define    mini_tcscat_s    mini_strcat_s
#define    mini_tcschr      mini_strchr
#define    mini_tcsrchr     mini_strrchr
#define    mini_tcstok_s    mini_strtok_s
#define    mini_tcstok      mini_strtok
#define    mini_vstprintf_s mini_vsprintf_s
#define    mini_vstprintf_size mini_vsprintf_size
#define    mini_stprintf_s  mini_sprintf_s
#define    mini_stprintf    mini_sprintf
#define    mini_ftprintf    mini_fprintf
#define    mini_tprintf     mini_printf
#endif

#ifndef MINICRT_BUILD

//
//  Have apps use them by default
//

#undef  _ttoi
#define _ttoi    mini_ttoi

#undef  atoi
#define atoi     mini_atoi

#undef  wtoi
#define wtoi     mini_wtoi

#undef  _tcschr
#define _tcschr  mini_tcschr

#undef  strchr
#define strchr   mini_strchr

#undef  wcschr
#define wcschr   mini_wcschr

#undef  _tcsrchr
#define _tcsrchr mini_tcsrchr

#undef  strrchr
#define strrchr  mini_strrchr

#undef  wcsrchr
#define wcsrchr  mini_wcsrchr

#undef  _tcsncat
#define _tcsncat mini_tcsncat

#undef  strncat
#define strncat  mini_strncat

#undef  wcsncat
#define wcsncat  mini_wcsncat

#undef  _tcscat_s
#define _tcscat_s mini_tcscat_s

#undef  strcat_s
#define strcat_s  mini_strcat_s

#undef  wcscat_s
#define wcscat_s  mini_wcscat_s

#undef  _tcscat
#define _tcscat(a,b) mini_tcscat_s(a, -1, b)

#undef  strcat
#define strcat(a,b)  mini_strcat_s(a, -1, b)

#undef  wcscat
#define wcscat(a,b)  mini_wcscat_s(a, -1, b)

#undef  _tcstok_s
#define _tcstok_s mini_tcstok_s

#undef  strtok_s
#define strtok_s  mini_strtok_s

#undef  wcstok_s
#define wcstok_s  mini_wcstok_s

#undef  _tcstok
#define _tcstok   mini_tcstok

#undef  strtok
#define strtok    mini_strtok

#undef  wcstok
#define wcstok    mini_wcstok

#undef  _memcpy
#define _memcpy mini_memcpy

#undef  memcpy
#define memcpy mini_memcpy

#undef  memset
#define memset mini_memset

#undef  memcmp
#define memcmp mini_memcmp

#undef  _stprintf_s
#define _stprintf_s  mini_stprintf_s

#undef  sprintf_s
#define sprintf_s    mini_sprintf_s

#undef  swprintf_s
#define swprintf_s   mini_swprintf_s

#undef  _sntprintf
#define _sntprintf   mini_stprintf_s

#undef  _snprintf
#define _snprintf    mini_sprintf_s

#undef  _snwprintf
#define _snwprintf   mini_swprintf_s

#undef  _stprintf
#define _stprintf    mini_stprintf

#undef  sprintf
#define sprintf      mini_sprintf

#undef  swprintf
#define swprintf     mini_swprintf

#undef  _ftprintf
#define _ftprintf    mini_ftprintf

#undef  fprintf
#define fprintf      mini_fprintf

#undef  fwprintf
#define fwprintf     mini_fwprintf

#undef  _tprintf
#define _tprintf     mini_tprintf

#undef  printf
#define printf       mini_printf

#undef  wprintf
#define wprintf      mini_wprintf

#undef  _tcscpy
#define _tcscpy(a,b) mini_stprintf_s(a,-1,_T("%s"),b)

#undef  strcpy
#define strcpy(a,b)  mini_sprintf_s(a,-1,"%s",b)

#undef  wcscpy
#define wcscpy(a,b)  mini_swprintf_s(a,-1,L"%s",b)

#endif // MINICRT_BUILD

//
//  Easy ones, just take them from Win32
//

#undef  _tcsicmp
#define _tcsicmp lstrcmpi

#undef  stricmp
#define stricmp lstrcmpiA

#undef  wcsicmp
#define wcsicmp lstrcmpiW

#undef  _tcscmp
#define _tcscmp lstrcmp

#undef  strcmp
#define strcmp  lstrcmpA

#undef  wcscmp
#define wcscmp  lstrcmpW

#undef  _tcslen
#define _tcslen lstrlen

#undef  strlen
#define strlen  lstrlenA

#undef  wcslen
#define wcslen  lstrlenW

#undef  malloc
#define malloc(x)   HeapAlloc(GetProcessHeap(), 0, x)

#undef  free
#define free(x)     HeapFree(GetProcessHeap(), 0, x)

#undef  exit
#define exit    ExitProcess

#undef stdin
#define stdin  0

#undef stdout
#define stdout 1

#undef stderr
#define stderr 2

